class NotKeyOptionException(Exception):
    
    """ Excepcion no existe configuraciones  """
    def __init__(self):
        super().__init__(
            'Service not foun or not have onjected in this handler'
        )


# -*- coding: utf-8 -*-
"""Implementacion del Framework"""

import importlib
import falcon
from und_microservice.handler.middlewares.basic import Basic
from und_microservice.commons.file import File

class FalconApi:
    """ Clase api, implementa el framework """

    def __init__(self):
        self.api = falcon.API(middleware=[
            Basic()
        ])
        self.__load_routes()

    def __load_routes(self):
        file_config = File.read_yml('bootstrap/routes.yml')
        prefix = file_config[0]['prefix']
        routes = file_config[1]['routes']
        for route in routes:
            for resource, handler in route.items():
                module_parts = handler.split('.')
                module_name = '.'.join(module_parts[:-1])
                module = importlib.import_module(module_name)
                handler = getattr(module, module_parts[-1])
                handler_instance = handler()
                self.api.add_route(prefix + resource, handler_instance)



# -*- coding: utf-8 -*-
"""Test module file"""

from und_microservice.commons.file import File

def test_yml_file():
    """Test read yml file"""
    espected_data = {
        'service': {
            'name': "x",
            'ip': "127.0.0.1",
            'user_name': "user"
        }
    }
    read_data = File.read_yml('test/resources/test_other_file.yml')
    assert espected_data == read_data

def test_json_file():
    """Test read json file"""
    espected_data = {
        "value": 1,
        "name": "example",
        "version": "1.0.1"
    }
    read_data = File.read_json('test/resources/test_file.json')
    assert espected_data == read_data

def test_txt_file():
    """Test read txt file"""
    espected_data = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed sem dolor'

    read_data = File.read_yml('test/resources/test_file.txt')
    assert espected_data == read_data



# -*- coding: utf-8 -*-
"""Test helper para datos genericos"""

from und_microservice.helper.data import dict_merge

def test_dict_merge():
    """Test para probar metodo para mesclar diccionarios"""
    dict_a = {
        'service': {
            'name': "x",
            'ip': "127.0.0.1",
            'user_name': "user"
        }
    }

    dict_b = {
        'db': {
            'connection': "mysql",
            'host': "localhost",
            'user': "root",
            'password': "password",
            'port': "3307",
            'database': "db",
            'lifetime': 180
        }
    }
    espected_dic = {
        'service': {
            'name': "x",
            'ip': "127.0.0.1",
            'user_name': "user"
        },
        'db': {
            'connection': "mysql",
            'host': "localhost",
            'user': "root",
            'password': "password",
            'port': "3307",
            'database': "db",
            'lifetime': 180
        },
    }

    dict_merge(dict_a, dict_b)
    assert dict_a == espected_dic



# -*- coding: utf-8 -*-
"""Test helper para url"""

from und_microservice.helper.url import get_filter_in

def test_split_query_string():
    """Test convertir querystring en array"""
    pass

def test_get_fileter_in():
    """Test para recuperar los filtros de una lista"""
    test_filters = {
        'index': [
            'i',
            'j'
        ],
        'index_b': 'name'
    }
    espected_list = {
        'index': [
            'i',
            'j'
        ]
    }
    result = get_filter_in(test_filters)
    assert espected_list == result

def test_join_query_string():
    """Union de query string"""
    pass



# -*- coding: utf-8 -*-
"""Test helper para datos genericos"""

from und_microservice.helper.string import concat

def test_concat_helper():
    """Prueba de concatenacion de elementos de un array"""
    test_data = [
        'salu',
        'do',
        'hola'
    ]
    espected_string = 'salu_do_hola'
    result = concat(test_data, '_')
    assert espected_string == result



# -*- coding: utf-8 -*-
"""Prueba del modulo conig"""

from und_microservice.config.config_yaml import ConfigYaml

def test_get_all():
    """Probando cargar todo los config"""
    config = ConfigYaml('test/resources/')
    espected_data = {
        'service': {
            'name': "x",
            'ip': "127.0.0.1",
            'user_name': "user"
        },
        'db': {
            'connection': "mysql",
            'host': "localhost",
            'user': "root",
            'password': "password",
            'port': "3307",
            'database': "db",
            'lifetime': 180
        }
    }
    all_data = config.get_all()
    assert espected_data == all_data

def test_get_key():
    """Test para recurar un valor grupo especifico"""
    config = ConfigYaml('test/resources/')
    espected_data = {
        'name': "x",
        'ip': "127.0.0.1",
        'user_name': "user"
    }
    service = config.get_key('service')
    assert service == espected_data



# -*- coding: utf-8 -*-
"""Modulo und_microservice.handler.decorator.service"""

from functools import wraps
import falcon


def service_validator(method):
    """Validar si existe un servicio"""

    @wraps(method)
    def method_wrapper(*args, **kwargs):
        """Wraper de decorador"""
        if args[0].service is None:
            raise falcon.HTTPError('Application Services is Null')
        return method(*args, **kwargs)
    return method_wrapper



import setuptools

try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements

from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
    long_description = long_description.replace("\r", "")  # YOU  NEED THIS LINE
except (OSError, ImportError):
    print("Pandoc not found. Long_description conversion failure.")
    import io
    # pandoc is not installed, fallback to using raw contents
    with io.open('README.md', encoding="utf-8") as f:
        long_description = f.read()

requirements = parse_requirements('requirements.txt', session=False)
install_requires = [str(r.req) for r in requirements]

setuptools.setup(
    name="und_microservice",
    author="UND",
    description="UND microservices common utilities",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/und-tech/und-python-microservice",
    install_requires = install_requires,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-
""" Excepcion base"""

class NotConfigException(Exception):
    """ Excepcion no existe configuraciones  """
    def __init__(self):
        super().__init__('Data not found ')

class NotValueConfigException(Exception):
    """ Excepcion no existe configuraciones  """
    def __init__(self, value):
        super().__init__('That key: ' + value + 'not found in config')



# -*- coding: utf-8 -*-
"""
    Modulo und_microservice.handler.base
    Base para handler y collectiones
"""

import falcon
from und_microservice.handler.decorator.service import service_validator
from und_microservice.helper import url
from und_microservice.response.orbis import Response
from und_microservice.handler.exception import ServiceNotExistError


class BaseCollectionHandler:
    """ Colleccion de handler """

    service = None
    response = Response()

    @service_validator
    def on_get(self, req, resp: falcon.Response):
        """ Default method GET """
        if self.service is None:
            raise ServiceNotExistError()
        try:
            params = url.split_query_string(req.params.items())
            resp.media = self.response.success(self.service.find_all(params))
            resp.status = falcon.HTTP_200
        except Exception as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500

    @service_validator
    def on_post(self, req: falcon.Request, resp: falcon.Response):
        """ Default method POST """
        try:
            kwargs = req.media['metadata']
            resp.media = self.response.success(self.service.create(**kwargs))
            resp.status = falcon.HTTP_200
        except Exception as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500

    @service_validator
    def on_put(self, req: falcon.Request, resp: falcon.Response):
        """ Default method PUT """
        try:
            kwargs = req.media['metadata']
            params = url.split_query_string(req.params.items())
            resp.media = self.response.success(self.service.update_collection(params, **kwargs))
            resp.status = falcon.HTTP_200
        except (Exception, AttributeError) as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500


class BaseHandler:
    """ Handler Base """
    service = None
    response = Response()

    @service_validator
    def on_get(self, req, resp: falcon.Response, id_value):
        """ Default method GET """
        try:
            resp.media = self.response.success(self.service.find_by_id(id_value))
            resp.status = falcon.HTTP_200
        except Exception as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500

    @service_validator
    def on_put(self, req: falcon.Request, resp: falcon.Response, id):
        """ Default method PUP """
        try:
            id = int(id) if id.isdigit() else id
            kwargs = req.media['metadata']
            if 'id' in kwargs:
                kwargs.pop('id')
            resp.media = self.response.success(self.service.update(id, **kwargs))
            resp.status = falcon.HTTP_200
        except (Exception, AttributeError) as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500

    @service_validator
    def on_delete(self, req, resp: falcon.Response, id):
        """ Default method DELETE """
        try:
            resp.media = self.response.success(self.service.delete(id))
            resp.status = falcon.HTTP_200
        except Exception as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500

    @service_validator
    def on_post(self, req: falcon.Request, resp: falcon.Response, id):
        """ Default method POST """
        try:
            id = int(id) if id.isdigit() else id
            kwargs = req.media['metadata']
            resp.media = self.response.success(self.service.create(id, **kwargs))
            resp.status = falcon.HTTP_200
        except Exception as exception:
            resp.media = self.response.error(exception.__str__())
            resp.status = falcon.HTTP_500



# -*- coding: utf-8 -*-
"""Modulo und_microservice.handler.midelwares.basic"""

import falcon

class Basic:
    """Clase midelware basico"""

    def process_request(self, req: falcon.Request, resp: falcon.Response):
        """Midelware de ejemplo"""
        print(req)
        print(resp)
        print('Im a basic middleware')
        return resp
